END_VIEW_NAME = "View"
INDEX_VIEW_PREFIX = "index"

def _get_module(view):

    module = view.__module__.split(".")[-1]
    return r"%s/" % module

def _get_name(view):

    if END_VIEW_NAME in view.__name__:
        return view.__name__[:-len(END_VIEW_NAME)].lower()
    return view.__name__.lower()

def _get_namespace(view):

    if hasattr(view, "namespace") and view.namespace is not None:
        return r"%s/" % view.namespace
    return ""

def get_template_name(view):
    
    namespace = _get_namespace(view)
    module = _get_module(view)
    name = _get_name(view)
    return "%s%s%s.html" % (namespace, module, name)

def get_url(view):

    url = r"^"
    url += _get_namespace(view)
    url += _get_module(view)

    name = _get_name(view)
    if name != INDEX_VIEW_PREFIX:
        url += r"%s/$" % name
    else:
        url += r"$"

    return url

def get_resource(view):

    return _get_name(view)


from introspector import Introspector


class UrlsManager(object):

    instrospector = Introspector()

    def __init__(self, urlpatterns, views_root, namespace=None):

        self.urlpatterns = urlpatterns
        self.views_root = views_root
        self.namespace = namespace

        self.add_views()

    def add_views(self):

        views = self.instrospector.infer_views(self.views_root)

        for view in views:

            if not "namespace" in view.__dict__ and self.namespace is not None:
                view.namespace = self.namespace

            django_urls = self.instrospector.get_django_urls(view)

            for url in django_urls:
                self.urlpatterns.append(url)



try:
    import simplejson as json
except ImportError:
    import json

from django.http import HttpResponse
from django.views.generic import View
from .. import conventions

from django.template.loader import get_template
from django.template import Context, Template


class RESTView(View):

    def response(self, response):

        return HttpResponse(response)

    def json_response(self, data):

        return HttpResponse(json.dumps(data))

    def render_to_response(self, template_name, data):

        return self.response(get_template(template_name).render(Context(data)))

    def get(self, *args, **kwargs):

        resource_id = kwargs.get("id")
        if not resource_id:
            return self.list(*args, **kwargs)

        return self.edit(*args, **kwargs)

    def post(self, *args, **kwargs):

        return self.create(*args, **kwargs)

    def put(self, *args, **kwargs):

        return self.update(*args, **kwargs)

    @classmethod
    def _infer_rest_urls(self):

        if "resource" in self.__dict__:
            resource = self.resource
        else:
            resource = conventions.get_resource(self)

        self.url = [r"^%s/(?P<id>\d)/$" % resource, r"^%s/$" % resource]
        return self